@@ -1,33 +1,49 @@
-use serde::de::DeserializeOwned;
-use serde::Serialize;
-
 use crate::checkout::Checkout;
 use crate::context::Context;
 use crate::customer::{Address, Contact};
 use crate::error::{new_not_found_error, Error};
 use crate::internal_context::InternalContext;
 use crate::money::Currency;
+use crate::payment::Payment;
 use crate::payment::PaymentProcessor;
 use crate::shipping::FulfillmentTypeSelection;
 
-pub async fn create_handler<T: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
-    ctx: &mut T,
+pub async fn create_handler<C: Context + Send>(
+    ctx: &mut C,
     currency: Currency,
     contact: Option<Contact>,
     shipping_address: Option<Address>,
-) -> Result<Checkout<P>, Error> {
+) -> Result<
+    Checkout<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let mut co = Checkout::new(ctx, currency, contact, shipping_address).await?;
     co.populate_associations(ctx).await?;
 
     Ok(co)
 }
 
-pub async fn add_item_handler<T: Context + Send, P: Sync + Send + Serialize + DeserializeOwned>(
-    ctx: &mut T,
+pub async fn add_item_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     sku: String,
     quantity: u64,
-) -> Result<Checkout<P>, Error> {
+) -> Result<
+    Checkout<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -43,15 +59,21 @@ pub async fn add_item_handler<T: Context + Send, P: Sync + Send + Serialize + De
     Ok(co)
 }
 
-pub async fn remove_item_handler<
-    T: Context + Send,
-    P: Sync + Send + Serialize + DeserializeOwned,
->(
-    ctx: &mut T,
+pub async fn remove_item_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     sku: String,
     quantity: u64,
-) -> Result<Checkout<P>, Error> {
+) -> Result<
+    Checkout<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -67,13 +89,19 @@ pub async fn remove_item_handler<
     Ok(co)
 }
 
-pub async fn confirm_items_handler<
-    T: Context + Send,
-    P: Sync + Send + Serialize + DeserializeOwned,
->(
-    ctx: &mut T,
+pub async fn confirm_items_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
-) -> Result<Checkout<P>, Error> {
+) -> Result<
+    Checkout<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -89,14 +117,20 @@ pub async fn confirm_items_handler<
     Ok(co)
 }
 
-pub async fn update_contact_handler<
-    T: Context + Send,
-    P: Sync + Send + Serialize + DeserializeOwned,
->(
-    ctx: &mut T,
+pub async fn update_contact_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     contact: Contact,
-) -> Result<Checkout<P>, Error> {
+) -> Result<
+    Checkout<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -112,14 +146,20 @@ pub async fn update_contact_handler<
     Ok(co)
 }
 
-pub async fn update_shipping_address_handler<
-    T: Context + Send,
-    P: Sync + Send + Serialize + DeserializeOwned,
->(
-    ctx: &mut T,
+pub async fn update_shipping_address_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     shipping_address: Address,
-) -> Result<Checkout<P>, Error> {
+) -> Result<
+    Checkout<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -135,14 +175,20 @@ pub async fn update_shipping_address_handler<
     Ok(co)
 }
 
-pub async fn update_fulfillment_type_handler<
-    T: Context + Send,
-    P: Sync + Send + Serialize + DeserializeOwned,
->(
-    ctx: &mut T,
+pub async fn update_fulfillment_type_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
     fulfillment_type: FulfillmentTypeSelection,
-) -> Result<Checkout<P>, Error> {
+) -> Result<
+    Checkout<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -158,14 +204,20 @@ pub async fn update_fulfillment_type_handler<
     Ok(co)
 }
 
-pub async fn initiate_payment_handler<
-    T: Context + Send,
-    P: Sync + Send + Serialize + DeserializeOwned,
->(
-    ctx: &mut T,
+pub async fn initiate_payment_handler<C: Context + Send>(
+    ctx: &mut C,
     checkout_id: &str,
-    args: <T as PaymentProcessor>::ProcessArgs,
-) -> Result<Checkout<P>, Error> {
+    args: <C as PaymentProcessor>::ProcessArgs,
+) -> Result<
+    Checkout<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
@@ -181,15 +233,21 @@ pub async fn initiate_payment_handler<
     Ok(co)
 }
 
-pub async fn initiate_order_handler<
-    T: Context + Send,
-    P: Sync + Send + Serialize + DeserializeOwned,
->(
+pub async fn initiate_order_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
-    ctx: &mut T,
+    ctx: &mut C,
     checkout_id: &str,
-    args: <T as PaymentProcessor>::ProcessArgs,
-) -> Result<Checkout<P>, Error> {
+    args: <C as PaymentProcessor>::ProcessArgs,
+) -> Result<
+    Checkout<
+        Payment<
+            <C as PaymentProcessor>::PrivateData,
+            <C as PaymentProcessor>::PublicData,
+            <C as PaymentProcessor>::ProcessArgs,
+        >,
+    >,
+    Error,
+> {
     let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
     if maybe_co.is_none() {
         return Err(new_not_found_error(
