@@ -44,6 +44,7 @@ pub struct Checkout<P> {
     pub order_id: Option<String>,
     // associations
     pub payment: Option<P>,
+    pub order: Option<Order<P>>,
 }
 
 #[async_trait]
@@ -89,6 +90,7 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
             order_id: None,
             // associations
             payment: None,
+            order: None,
         };
 
         ctx.create_checkout(&co).await?;
@@ -486,6 +488,14 @@ impl<P: Sync + Send + Serialize + DeserializeOwned> Checkout<P> {
             self.payment = ctx.get_payment(payment_id).await?;
         }
 
+        if let Some(ref order_id) = self.order_id {
+            let mut maybe_order: Option<Order<P>> = ctx.get_order(order_id).await?;
+            if let Some(ref mut order) = maybe_order {
+                order.populate_associations(ctx).await?;
+            }
+            self.order = maybe_order;
+        }
+
         Ok(())
     }
 }
