@@ -5,7 +5,6 @@ use serde::{Deserialize, Serialize};
 use std::str::FromStr;
 
 use crate::checkout::Checkout;
-use crate::context::Context;
 use crate::error::Error;
 use crate::money::Money;
 use crate::shipping::FulfillmentType;
@@ -41,30 +40,30 @@ pub struct Invoice {
 
 #[async_trait]
 pub trait InvoiceCalculator {
-    async fn line_item_taxes<C: Context + Send>(
+    async fn line_item_taxes<P: Sync + Send>(
         &mut self,
-        _co: &Checkout<C>,
+        _co: &Checkout<P>,
     ) -> Result<Vec<Tax>, Error> {
         Ok(vec![])
     }
 
-    async fn shipping_taxes<C: Context + Send>(
+    async fn shipping_taxes<P: Sync + Send>(
         &mut self,
-        _co: &Checkout<C>,
+        _co: &Checkout<P>,
     ) -> Result<Vec<Tax>, Error> {
         Ok(vec![])
     }
 
-    async fn initial_charge_ratio<C: Context + Send>(
+    async fn initial_charge_ratio<P: Sync + Send>(
         &mut self,
-        _co: &Checkout<C>,
+        _co: &Checkout<P>,
     ) -> Result<Decimal, Error> {
         Ok(Decimal::from_str("1").unwrap())
     }
 
-    async fn generate_invoice<C: Context + Send>(
+    async fn generate_invoice<P: Sync + Send>(
         &mut self,
-        co: &Checkout<C>,
+        co: &Checkout<P>,
     ) -> Result<Invoice, Error> {
         let line_items: Vec<LineItem> = co
             .items
